package com.perforce.hive.artifact.proxy;

import com.perforce.hive.PluginArtifactFactory;
import com.perforce.hive.api.artifact.ArtifactInterface;
import com.perforce.hive.artifact.DefaultArtifact;
import com.perforce.hive.depot.DepotFactory;
import com.perforce.hive.depot.DepotModel;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.server.IOptionsServer;
import org.apache.commons.fileupload.FileUploadException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import spark.Request;

import java.io.File;
import java.io.InputStream;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

public class ProxyArtifact extends DefaultArtifact implements ArtifactInterface {

	private static Logger logger = LoggerFactory.getLogger(ProxyArtifact.class);

	private URL proxy;

	public ProxyArtifact(IOptionsServer p4, URL proxy, String depot, String path) throws Exception {
		super(p4);

		this.proxy = proxy;
		this.path = Paths.get(path);
		this.repoID = depot;
		configureClient();
	}

	@Override
	public Path getPath() {
		String groupID = path.getParent().toString();
		String id[] = {groupID};
		Path depotPath = Paths.get(repoID, id);
		return depotPath;
	}

	@Override
	public DepotModel getDepot() throws P4JavaException {
		DepotModel depot = DepotFactory.getByName(p4, repoID);
		return depot;
	}

	@Override
	public List<String> populateMultiPartPost() throws Exception {

		File root = new File(getClient().getRoot());

		List<String> list = new ArrayList<>();
		URL url = new URL(proxy, path.toString());
		Path filename = path.getFileName();
		Path filepath = Paths.get(root.toString(), filename.toString());

		logger.info("[proxy] caching: " + url + " (" + filepath + ")");
		InputStream in = url.openStream();
		Files.copy(in, filepath);
		in.close();

		list.add(filepath.toString());
		return list;
	}

	@Override
	public void processForm(Request request) throws FileUploadException {
		// No form to process
	}

	@Override
	public void index() throws Exception {
		ArtifactInterface artifact = PluginArtifactFactory.getProxy(p4, repoID);
		artifact.index();
		if (artifact instanceof DefaultArtifact) {
			DefaultArtifact defaultArtifact = (DefaultArtifact) artifact;
			logger.info("CLOSE " + defaultArtifact.getClient().getName());
			defaultArtifact.close();
		}
	}

	@Override
	public void buildIndex() throws Exception {
		// no indexing
	}
}
