package com.perforce.hive.browse;

import java.util.Map;

import spark.ModelAndView;
import spark.Request;
import spark.Response;

import com.perforce.hive.ErrorModel;
import com.perforce.hive.connection.ConnectionSession;
import com.perforce.hive.depot.DepotFactory;
import com.perforce.hive.navigation.NavBar;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.server.IOptionsServer;

public class BrowseControl extends ErrorModel {

	public static ModelAndView browse(Request request, Response response) {
		Map<String, Object> model;
		model = NavBar.attributes("browse", request);

		String base = request.pathInfo();
		base = (base.endsWith("/")) ? base : base + "/";

		BreadcrumbModel breadCrumb = new BreadcrumbModel(base);
		model.put("breadcrumb", breadCrumb.getList());

		try {
			IOptionsServer p4 = ConnectionSession.get(request, response);
			BrowseModel browse = new BrowseModel(p4, base);
			model.put("directories", browse.getDirs());
			model.put("files", browse.getFiles());
			model.put("summary", browse.getSummary());
			model.put("base", browse.getBase());
			model.put("depot", DepotFactory.getByBase(p4, base));

			p4.disconnect();
		} catch (P4JavaException e) {
			return error(e);
		}

		return new ModelAndView(model, "/browse/page-items.html");
	}

	public static ModelAndView depots(Request request, Response response) {
		Map<String, Object> model;
		model = NavBar.attributes("browse", request);

		try {
			IOptionsServer p4 = ConnectionSession.get(request, response);
			model.put("depots", DepotFactory.listAll(p4));
		} catch (P4JavaException e) {
			return error(e);
		}

		return new ModelAndView(model, "/browse/page-depots.html");
	}

	public static ModelAndView list(Request request, Response response) {
		Map<String, Object> model;
		model = NavBar.attributes("browse", request);
		model.put("sort", request.queryParams("sort"));

		try {
			IOptionsServer p4 = ConnectionSession.get(request, response);
			model.put("depots", DepotFactory.hashListAll(p4));
		} catch (P4JavaException e) {
			return error(e);
		}

		return new ModelAndView(model, "/browse/page-list.html");
	}
}